import { Observable, Subject } from 'rxjs';

import { Message } from '../../../core/type';
import {
  RxWebSocketSubject,
  RxWebSocketSubjectConfig
} from '../../../dom/rx_websocket_subject';
import { ClientRWC } from '../../client_rwc';

export interface WebSocketClientRWCConfig {
  url: string;
  protocol?: string | Array<string>;
  WebSocketCtor?: { new(url: string, protocols?: string | string[]): WebSocket };
  queryString?: string;
  reconnectInterval?: 5000;
  reconnectRetry?: 10;
}

export class WebSocketClientRWC implements ClientRWC {
  private wsSubject: RxWebSocketSubject<Message> | undefined;
  private _rxConfig: RxWebSocketSubjectConfig<Message>;
  private resSubject: Subject<Message> | undefined;

  constructor(private _config: WebSocketClientRWCConfig) {
    this._rxConfig = Object.assign({}, this._config);
    // this._rxConfig.resultSelector = (e: MessageEvent) => {
    //   return e.data;
    // };
    this._rxConfig.serializer = (value: Message) => value;
    this._rxConfig.deserializer = (e: MessageEvent) => e.data;
    this._rxConfig.binaryType = 'arraybuffer';
  }

  public connect(queryString?: string): void {
    if (undefined === this.wsSubject) {
      if (undefined !== queryString) {
        this._rxConfig.queryString = queryString;
      }
      this.wsSubject = new RxWebSocketSubject(this._rxConfig);
    }
    this.wsSubject.subscribe(
      (value: Message) => {
        if (undefined !== this.resSubject) {
          this.resSubject.next(value);
        }
      },
      (error: any) => {
        if (undefined !== this.resSubject) {
          this.resSubject.error(error);
        }
      },
      () => {
        console.log('sss');
      },
    );
  }

  public disconnect(): void {
    if (undefined !== this.wsSubject) {
      this.wsSubject.disconnect();
    }
  }

  public connectionStatus(): Observable<boolean> | undefined {
    if (undefined !== this.wsSubject) {
      return this.wsSubject.connectionStatus;
    }

    return undefined;
  }

  public read(): Observable<Message> {
    if (undefined === this.resSubject) {
      this.resSubject = new Subject<Message>();
    }

    return this.resSubject.asObservable();
  }

  public write(data: Message): void {
    if (undefined !== this.wsSubject) {
      this.wsSubject.send(data);
    }
  }
}
